// GENFSTAB

use crate::{
    config::GeneralConfig,
    linux::{arch_chroot, run_command, systemd_service_enable},
    print_status,
};

use super::{uncomment_first_value_of, user::change_passwd};

/// Generate the `/etc/fstab` file
pub fn genfstab() {
    print_status("Generating fstab");
    let (stdout, _) = run_command(&["genfstab", "-U", "/mnt"], None, false);
    std::fs::write("/mnt/etc/fstab", stdout).unwrap();
}

/// Set common config values for the system
pub fn first_boot_values(conf: &GeneralConfig) {
    // Locale
    print_status(&format!("Setting locale {}", conf.locale));
    std::fs::write(
        "/mnt/etc/locale.conf",
        format!("LANG=\"{}\"\n", conf.locale),
    )
    .unwrap();

    // Timezone
    print_status(&format!("Setting timezone {}", conf.timezone));
    let tz_paths = conf.timezone.split('/').collect::<Vec<&str>>();
    let mut tz_link = std::path::PathBuf::from("/usr/share/zoneinfo/");

    for path in tz_paths {
        tz_link.push(path);
    }

    std::os::unix::fs::symlink(tz_link, "/mnt/etc/localtime").unwrap();

    // Keymap
    print_status("Writing /etc/vconsole.conf");
    std::fs::write("/mnt/etc/vconsole.conf", build_vconsole_conf(conf)).unwrap();

    // Hostname
    print_status("Writing /etc/hostname");
    std::fs::write("/mnt/etc/hostname", format!("{}\n", conf.hostname)).unwrap();

    // LOCALE
    print_status("Setting locale");
    uncomment_first_value_of(&conf.locale, "/mnt/etc/locale.gen");
    arch_chroot(&["locale-gen"], None, false);

    arch_chroot(&["hwclock", "--systohc"], None, false);

    if let Some(root_pw) = &conf.root_password {
        change_passwd("root", root_pw);
    }

    systemd_service_enable("NetworkManager.service");
}

pub fn build_vconsole_conf(conf: &GeneralConfig) -> String {
    let mut ret = format!("KEYMAP={}\n", conf.keyboard_layout);
    ret.push_str(&format!("XBKLAYOUT={}\n", conf.keyboard_layout));

    if let Some(variant) = &conf.keyboard_variant {
        ret.push_str(&format!("XKBMODEL={variant}\n"));
    }

    ret
}
